<?php
// $Id: views_handler_filter_dynamic_fields.inc,v 1.5 2010/10/25 10:10:12 girishmuraly Exp $
/**
 * @file
 * Views dynamic fields filter handler
 */

/**
 * Simple filter to pick and choose the fields to show in the view.
 */
class views_handler_filter_dynamic_fields extends views_handler_filter {

  var $no_single = TRUE;

  function has_extra_options() { return TRUE; }

  function option_definition() {
    $options = parent::option_definition();
    $options['filterable_fields'] = array('default' => array());
    $options['reverse'] = array('default' => 0);
    $options['reverse_label'] = array('default' => '');
    return $options;
  }

  /**
   * Provide a list of all fields that are for display
   */
  function value_form(&$form, &$form_state) {
    $form['value']['#tree'] = TRUE;
    $all_fields = array();
    // Get all the fields defined in this view
    $view_fields = $this->view->get_items('field');

    // Remove fields that have been combined with other fields
    // E.g. using table style plugin, one can set the column of one field as that of another so they appear as one column in the output
    $style_columns = $this->view->display_handler->default_display->options['style_options']['columns'];
    $combined_fields = array();
    if (is_array($style_columns)) {
      foreach ($style_columns as $key => $name) {
        if ($view_fields[$key] && $key != $name) {
          unset($view_fields[$key]);
          // Capture this information for processing in pre_query()
          // The $key field will be acted upon if the $name (parent) field is acted upon by the user
          $combined_fields[$name] = $key;
        }
      }
    }

    $filterable_fields = array_filter($this->options['filterable_fields']);

    foreach ($view_fields as $field_name => $field) {
      // Apply filterable fields options
      if (!empty($filterable_fields) && !isset($filterable_fields[$field_name])) {
        continue;
      }

      if (!isset($_SESSION[$this->view->name]['perm_exc']) && $field['exclude']) {
        // Fields already marked as 'exclude from display' in the original view should be omitted
        $_SESSION[$this->view->name]['perm_exc'][$field_name] = $field_name;
        continue;
      }
      if (!isset($_SESSION[$this->view->name]['perm_exc'][$field_name])) {
        if (empty($field['label'])) {
          $all_fields[$field_name] = $field['id'];
        }
        else {
          $all_fields[$field_name] = $field['label'];
        }
        $field_names[] = $field_name;
      }
    }
    // Initialize the default exclusion array if not set
    if (!isset($_SESSION[$this->view->name]['perm_exc'])) {
      $_SESSION[$this->view->name]['perm_exc'] = array();
    }

    $this->options['expose']['single'] = ''; // otherwise views_handler_filter will change this to use 'select'
    if($form_state['exposed']) {
      $form['value'] = array(
        '#type' => 'checkboxes',
        '#no_convert' => TRUE,
        '#title' => $this->options['expose']['label'] ? t($this->options['expose']['label']) : t('Fields'),
        '#default_value' => $field_names,
        '#options' => $all_fields,
        '#description' => t('Select fields for the view - these will be shown by default'),
      );
      $form['combined'] = array(
        '#type' => 'hidden',
        '#value' => serialize($combined_fields),
        '#weight' => 9,
      );
      if ($this->options['reverse']) {
        $form['reverse'] = array(
          '#type' => 'checkbox',
          '#default_value' => $this->options['reverse'],
          '#title' => $this->options['reverse_label'] ? t($this->options['reverse_label']) : t('Show fields to be excluded'),
          '#description' => t('Use the field selection to exclude them from the view'),
        );
      }
    }
    else {
      // Ensure there is something in the 'value'.
      $form['value'] = array(
        '#type' => 'value',
        '#value' => NULL
      );
    }
  }

  /**
   * Provide an 'exclusion mode' option
   * @see sites/all/modules/contrib/views/includes/views_handler#extra_options_form()
   */
  function extra_options_form(&$form, &$form_state) {
    // Get all non-excluded fields
    $view_fields = $this->view->get_items('field');
    foreach ($view_fields as $field_name => $field) {
      if (empty($field['label'])) {
        $field_label = $field['id'];
      }
      else {
        $field_label = $field['label'];
      }
      // Do not allow pre-configured excluded fields in the filter
      if (!$field['exclude']) {
        $fields[$field_name] = $field_label;
      }
      else {
        $fields_excluded[$field_name] = $field_label;
      }
    }

    $form['filterable_fields'] = array(
      '#type' => 'checkboxes',
      '#title' => t('Fields'),
      '#description' => t('Select the fields that you want to be available for filtering. If no field is selected, all filters will be available.'),
      '#options' => $fields,
      '#default_value' => $this->options['filterable_fields'],
    );

    if ($fields_excluded) {
      $form['filterable_fields_excluded'] = array(
      	'#type' => 'checkboxes',
        '#title' => 'Excluded fields',
      	'#options' => $fields_excluded,
        '#description' => t('You have configured these fields to be excluded from the view, hence they will not appear as filters.'),
        '#disabled' => TRUE,
      );
    }

    $form['reverse'] = array(
      '#type' => 'checkbox',
      '#default_value' => $this->options['reverse'],
      '#title' => t('Provide exclusion mode'),
      '#description' => t('Provide a checkbox to reverse the selection from inclusion to exclusion'),
    );
    $form['reverse_label'] = array(
      '#type' => 'textfield',
      '#default_value' => $this->options['reverse_label'],
      '#title' => t('Label for exclusion fields options'),
      '#size' => 40,
    );
  }

  /**
   * Exclude fields from the view before the query is run
   * @see sites/all/modules/contrib/views/includes/views_handler#pre_query()
   */
  function pre_query() {
    $exposed_input = $this->view->get_exposed_input();

    $combined_fields = unserialize($exposed_input['combined']);

    // Get all fields
    foreach ($this->view->field as $field_name => $field) {
      $field_names[] = $field_name;
    }
    if (!empty($exposed_input[$this->options['expose']['identifier']])) {
      // Exclude these fields

      $extra_exc = array(); // Holder for any combined fields we need to remove if their parent combined field is removed

      // If exclusion mode is selected
      if ($_REQUEST['reverse']) {
        $fields_to_exclude = $exposed_input[$this->options['expose']['identifier']];
      }
      else {
        $fields_to_exclude = array_diff($field_names, $exposed_input[$this->options['expose']['identifier']]);
      }

      $filterable_fields = array_filter($this->options['filterable_fields']);

      // Also include/exclude the fields combined into any of the $exposed_input fields
      foreach($fields_to_exclude as $key => $efield) {
        // Add the combined field to the exclusion array if parent is filterable and excluded
        if (isset($combined_fields[$efield]) && $this->options['filterable_fields'][$efield] !== 0) {
          $fields_to_exclude[$combined_fields[$efield]] = $combined_fields[$efield];
        }
        // On the other hand, the combined field should be include in the view if the parent is
        if (in_array($efield, $combined_fields)) {
          unset($fields_to_exclude[$key]);
        }
      }

      // Mark the fields to be excluded to the view object
      foreach ($fields_to_exclude as $field_to_exclude) {
        // Exclude only the fields chosen as excludable in filter options
        if (empty($filterable_fields) || $this->options['filterable_fields'][$field_to_exclude] !== 0) {
          $this->view->field[$field_to_exclude]->options['exclude'] = 1;
        }
      }
    }
  }

  /**
   * Do nothing in the query. This function needs to be defined and left to do nothing.
   * @see sites/all/modules/contrib/views/handlers/views_handler_filter#query()
   */
  function query() {
  }

}
